import asyncio
from datetime import datetime, timedelta
from telegram import Update
from telegram.ext import (
    Application,
    CommandHandler,
    MessageHandler,
    filters,
    ContextTypes
)
import aiohttp
import nest_asyncio
from functools import wraps


# BOT_TOKEN = "7664853316:AAEP2dAb3GuWLib6eJB1vu8DIR65HqlIBRA"
# KEYWORD_LIST_URL = "https://loopaddon.uk/files/PJ/BOT_DATA/loop_bot_keywords.json"

# BOT_TOKEN = "YOUR_BOT_TOKEN_HERE"  # Replace with your actual token
# Loop Gatekeeper bot
BOT_TOKEN = "7664853316:AAEP2dAb3GuWLib6eJB1vu8DIR65HqlIBRA"
# Loop Development
# BOT_TOKEN = "7309367551:AAGJNNPPQroL7yqw7U7eeUmsOXrKVQgzAw0" 
KEYWORD_LIST_URL = "https://loopaddon.uk/files/PJ/BOT_DATA/loop_bot_keywords.json"

new_users = {}
user_warnings = {}

check_list1 = []
check_list2 = []
check_list3 = []
allowed_users = []

import json
import os

PERSIST_FILE = "new_users.json"

def save_new_users():
    with open(PERSIST_FILE, "w") as f:
        json.dump(new_users, f, default=str)

def load_new_users():
    global new_users
    if os.path.exists(PERSIST_FILE):
        with open(PERSIST_FILE, "r") as f:
            raw = json.load(f)
            # Convert join_time back to datetime
            new_users = {
                int(uid): {
                    "chat_id": data["chat_id"],
                    "join_time": datetime.fromisoformat(data["join_time"])
                }
                for uid, data in raw.items()
            }
    else:
        new_users = {}  # Initialize as empty if the file is missing            

def admin_or_allowed_user(func):
    @wraps(func)
    async def wrapper(update: Update, context: ContextTypes.DEFAULT_TYPE, *args, **kwargs):
        user = update.effective_user
        chat = update.effective_chat

        # Case: Message sent by an anonymous admin (sender_chat is the group itself)
        if update.message.sender_chat and update.message.sender_chat.id == chat.id:
            return await func(update, context, *args, **kwargs)

        # Regular user case
        is_admin = await is_user_admin(chat, user.id)
        current_allowed_users = set(allowed_users)
        is_allowed_user = (
            is_admin or 
            str(user.id) in current_allowed_users or 
            (user.username and f"@{user.username.lower()}" in [u.lower() for u in current_allowed_users])
        )

        if not is_allowed_user:
            await update.message.reply_text("You need to be an admin or an authorized user to use this command.")
            return

        return await func(update, context, *args, **kwargs)
    return wrapper

async def is_user_admin(chat, user_id):
    administrators = await chat.get_administrators()
    return any(admin.user.id == user_id for admin in administrators)

async def fetch_keyword_lists():
    global check_list1, check_list2, check_list3, allowed_users
    try:
        async with aiohttp.ClientSession() as session:
            async with session.get(KEYWORD_LIST_URL) as response:
                if response.status == 200:
                    data = await response.json()
                    check_list1 = data.get("check_list1", [])
                    check_list2 = data.get("check_list2", [])
                    check_list3 = data.get("check_list3", [])
                    allowed_users = data.get("allowed_users", [])
                    print("Keyword lists updated successfully.")
                    print("\n" + f"Allowed users: {allowed_users}")
                else:
                    print(f"Failed to fetch keyword lists. HTTP Status: {response.status}")
    except Exception as e:
        print(f"Error fetching keyword lists: {e}")


@admin_or_allowed_user
async def manual_refresh_keywords(update: Update, context: ContextTypes.DEFAULT_TYPE):
    await fetch_keyword_lists()
    await update.message.reply_text("Keyword lists refreshed manually.")


@admin_or_allowed_user
async def start(update, context):
    await update.message.reply_text(
        "Gate Keeper is active \n\n"
        "Be aware there is a 3 strike warning process in place. \n\n"
        "Any user that receives 3 warnings will be automatically removed from the group."
    )


@admin_or_allowed_user
async def warn_user(update, context):
    if update.message.reply_to_message:
        warned_user = update.message.reply_to_message.from_user
        target_id = warned_user.id
        chat_id = update.message.chat.id

        user_warnings[target_id] = user_warnings.get(target_id, 0) + 1
        warnings = user_warnings[target_id]

        await update.message.reply_text(
            f"*{warned_user.first_name}* has been warned\\. \nTotal warnings: {warnings}\\.",
            parse_mode='MarkdownV2'
        )

        if warnings >= 3:
            try:
                await context.bot.ban_chat_member(chat_id, target_id)
                await context.bot.send_message(
                    chat_id, f"*{warned_user.first_name}* has been removed for receiving 3 warnings\\.",
                    parse_mode='MarkdownV2'
                )
                del user_warnings[target_id]
            except Exception as e:
                await update.message.reply_text(f"Failed to remove *{warned_user.first_name}*\\. Error: {e}", parse_mode='MarkdownV2')
    else:
        await update.message.reply_text("Please reply to a user's message to warn them.")


@admin_or_allowed_user
async def reset_warnings(update, context):
    if update.message.reply_to_message:
        warned_user = update.message.reply_to_message.from_user
        target_id = warned_user.id

        if target_id in user_warnings:
            del user_warnings[target_id]
            await update.message.reply_text(f"Warnings for *{warned_user.first_name}* have been reset\\.", parse_mode='MarkdownV2')
        else:
            await update.message.reply_text(f"*{warned_user.first_name}* has no warnings\\.", parse_mode='MarkdownV2')
    else:
        await update.message.reply_text("Please reply to a user's message to reset their warnings.")


@admin_or_allowed_user
async def ban_user(update, context):
    if update.message.reply_to_message:
        banned_user = update.message.reply_to_message.from_user
        target_id = banned_user.id
        chat_id = update.message.chat.id

        reason = " ".join(context.args) if context.args else "No reason provided."

        try:
            await context.bot.ban_chat_member(chat_id, target_id)
            await context.bot.send_message(
                chat_id, f"*{banned_user.first_name}* has been banned\\. \nReason: {reason}",
                parse_mode='MarkdownV2'
            )
        except Exception as e:
            await update.message.reply_text(f"Failed to ban *{banned_user.first_name}*\\. Error: {e}", parse_mode='MarkdownV2')
    else:
        await update.message.reply_text("Please reply to a user's message to ban them.")

@admin_or_allowed_user
async def add_profile_pic(update, context):
    chat_id = update.effective_chat.id
    user = None

    if update.message.reply_to_message:
        user = update.message.reply_to_message.from_user
    else:
        await update.message.reply_text("Please reply to the user's message to check their profile picture.")
        return

    first_name = user.first_name

    try:
        user_photos = await context.bot.get_user_profile_photos(user.id)
        if user_photos.total_count == 0:
            await context.bot.send_photo(
                chat_id=chat_id,
                photo='https://i.imgur.com/q3l7yOM.jpeg',
                caption=(
                    f"*{first_name}*\n\n"
                    "Please ensure you have a profile picture as it's a requirement of this group\\. "
                    "You will be automatically removed from this group if you do not have a profile picture within 24 hours\\."
                ),
                parse_mode='MarkdownV2'
            )
        else:
            await update.message.reply_text(
                f"*{first_name}* already has a profile picture set\\. 👍",
                parse_mode='MarkdownV2'
            )
    except Exception as e:
        await update.message.reply_text(f"Error fetching profile picture: {e}")

#########      
  
async def welcome_new_user(update, context):
    if update.message.new_chat_members:
        for user in update.message.new_chat_members:
            user_id = user.id
            chat_id = update.message.chat.id
            first_name = user.first_name
            
            name = user.username or user.first_name

            try:
                user_photos = await context.bot.get_user_profile_photos(user_id)
                pic_count = user_photos.total_count
            except:
                pic_count = 0

            if pic_count == 0:
                await context.bot.send_photo(
                    chat_id=chat_id,
                    photo='https://i.imgur.com/q3l7yOM.jpeg',
                    caption=(
                        f"Welcome *{name}*\n\n"
                        "Please ensure you have a profile picture as it's a requirement of this group\\. "
                        "You will be automatically removed from this group if you do not have a profile picture within 24 hours of joining\\."
                    ),
                    parse_mode='MarkdownV2'
                )
            else:
                await context.bot.send_message(
                    chat_id=chat_id,
                    text=(
                        f"Welcome *{name}*\n\n"
                        "Please see pinned post for installation instructions\\."
                    ),
                    parse_mode='MarkdownV2'
                )

            new_users[user_id] = {"chat_id": chat_id, "join_time": datetime.now()}
            
            save_new_users()

async def handle_message(update, context):
    if not update.message or not update.message.text:
        return

    text = update.message.text.lower()
    user_id = update.message.from_user.id
    first_name = update.message.from_user.first_name
    username = update.message.from_user.username or first_name  # Fallback to first_name if no username
    chat_id = update.message.chat.id

    # Fetch user profile pictures
    try:
        user_photos = await context.bot.get_user_profile_photos(user_id)
        pic_count = user_photos.total_count
    except:
        pic_count = 0  # If there's an error, assume no profile picture

    # If no profile picture, notify and issue warning
    if pic_count == 0:
        # Notify user that their comment will be responded to only after setting a profile picture
        await update.message.reply_text(
            f"@{username} Your comment has been received and will be responded to once you have a profile picture\\.",
            parse_mode='MarkdownV2'
        )

        # Increment warnings count for the user
        user_warnings[user_id] = user_warnings.get(user_id, 0) + 1
        warnings = user_warnings[user_id]

        # Send a warning message
        await update.message.reply_text(
            f"Warning *{first_name}* \nThis is warning {warnings}\\. You need a profile picture to receive further responses\\.",
            parse_mode='MarkdownV2'
        )

        # If user reaches 3 warnings, ban them
        if warnings >= 3:
            try:
                await context.bot.ban_chat_member(chat_id, user_id)
                await context.bot.send_message(
                    chat_id, f"*{first_name}* has been removed for receiving 3 warnings\\.",
                    parse_mode='MarkdownV2'
                )
                del user_warnings[user_id]  # Reset warnings after ban
            except Exception as e:
                await update.message.reply_text(f"Failed to remove *{first_name}*\\. Error: {e}", parse_mode='MarkdownV2')

async def remove_users_without_profile_pics(context: ContextTypes.DEFAULT_TYPE):
    now = datetime.now()
    bot = context.bot
    to_remove = []

    for user_id, data in new_users.items():
        join_time = data['join_time']
        chat_id = data['chat_id']

        if (now - join_time).total_seconds() >= 86400:
            try:
                user_photos = await bot.get_user_profile_photos(user_id)
                if user_photos.total_count == 0:
                    await bot.ban_chat_member(chat_id, user_id)
                    await bot.send_message(
                        chat_id,
                        f"User ID {user_id} has been removed for not setting a profile picture\\.",
                        parse_mode='MarkdownV2'
                    )
                to_remove.append(user_id)
            except Exception as e:
                print(f"Error checking/removing user {user_id}: {e}")

    for uid in to_remove:
        del new_users[uid]
    
    save_new_users()

@admin_or_allowed_user
async def preview_no_profile_pics(update: Update, context: ContextTypes.DEFAULT_TYPE):
    bot = context.bot

    if not new_users:
        await update.message.reply_text("No new users are currently being tracked.")
        return

    users_to_remove = []

    for user_id, data in new_users.items():
        chat_id = data['chat_id']
        join_time = data['join_time']
        try:
            member = await bot.get_chat_member(chat_id, user_id)
            user_photos = await bot.get_user_profile_photos(user_id)

            if user_photos.total_count == 0:
                # Calculate time spent in group
                time_in_group = datetime.now() - join_time
                hours_in_group = time_in_group.total_seconds() // 3600
                minutes_in_group = (time_in_group.total_seconds() % 3600) // 60

                display_name = f"@{member.user.username}" if member.user.username else f"ID: `{user_id}`"
                users_to_remove.append((display_name, hours_in_group, minutes_in_group))
        except Exception as e:
            print(f"Error previewing user {user_id}: {e}")
            continue

    if users_to_remove:
        # Sort users by join time (longest time spent first)
        users_to_remove.sort(key=lambda x: (x[1], x[2]), reverse=True)

        preview_text = (
            "The following users **would be removed** for not having a profile picture:\n\n"
        )
        for display_name, hours, minutes in users_to_remove:
            preview_text += (
                f"• {display_name} (joined {int(hours)} hours and {int(minutes)} minutes ago)\n"
            )

        await update.message.reply_text(preview_text, parse_mode="Markdown")
    else:
        await update.message.reply_text("All tracked users currently have profile pictures.")

@admin_or_allowed_user
async def manual_check_no_profile_pics(update: Update, context: ContextTypes.DEFAULT_TYPE):
    now = datetime.now()
    bot = context.bot
    removed_count = 0
    to_remove = []

    for user_id, data in new_users.items():
        join_time = data['join_time']
        chat_id = data['chat_id']

        try:
            user_photos = await bot.get_user_profile_photos(user_id)
            if user_photos.total_count == 0:
                await bot.ban_chat_member(chat_id, user_id)
                await bot.send_message(
                    chat_id,
                    f"User ID {user_id} has been removed manually for not setting a profile picture.",
                    parse_mode='MarkdownV2'
                )
                to_remove.append(user_id)
                removed_count += 1
        except Exception as e:
            print(f"Error manually removing user {user_id}: {e}")

    for uid in to_remove:
        del new_users[uid]

    save_new_users()

    await update.message.reply_text(f"Manual check complete. Removed {removed_count} users without profile pictures.")

async def run_bot():
    application = Application.builder().token(BOT_TOKEN).build()
    
    # Load new_users from disk
    load_new_users()
    
    # Fetch keywords and allowed users before registering handlers
    await fetch_keyword_lists()

    # Register handlers AFTER keyword list is ready
    application.add_handler(CommandHandler("start", start))
    application.add_handler(CommandHandler("warn", warn_user))
    application.add_handler(CommandHandler("reset", reset_warnings))
    application.add_handler(CommandHandler("ban", ban_user))
    application.add_handler(CommandHandler("profile", add_profile_pic))
    application.add_handler(CommandHandler("refresh", manual_refresh_keywords))
    application.add_handler(CommandHandler("clean", manual_check_no_profile_pics))
    application.add_handler(CommandHandler("scan", preview_no_profile_pics))
    
    application.add_handler(MessageHandler(filters.StatusUpdate.NEW_CHAT_MEMBERS, welcome_new_user))
    application.add_handler(MessageHandler(filters.TEXT & ~filters.COMMAND, handle_message))

    # application.job_queue.run_repeating(lambda _: asyncio.create_task(fetch_keyword_lists()), interval=7200)
    application.job_queue.run_repeating(fetch_keyword_lists, interval=7200)
    application.job_queue.run_repeating(remove_users_without_profile_pics, interval=86400)

    print("\n"+"Bot is running..."+"\n")
    await application.run_polling()

def main():
    nest_asyncio.apply()
    asyncio.get_event_loop().run_until_complete(run_bot())

if __name__ == "__main__":
    main()